/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package bo.gob.adsib.validador.test.itext2;

import com.itextpdf.signatures.DigestAlgorithms;
import com.itextpdf.signatures.IExternalSignature;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.Provider;
import java.security.Security;
import java.security.Signature;
import java.security.UnrecoverableKeyException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;

/**
 *
 * @author alberto
 */
public class ServerSignature implements IExternalSignature {
    protected String pin;
    protected String alias;
    protected Provider sunPKCS11;
    protected KeyStore keyStore;

    public ServerSignature(String pin, String alias) {
        this.pin = pin;
        this.alias = alias;

        sunPKCS11 = Security.getProvider("SunPKCS11");
        sunPKCS11 = sunPKCS11.configure("token.cfg");
        try {
            keyStore = KeyStore.getInstance("PKCS11", sunPKCS11);
            keyStore.load(null, pin.toCharArray());
        } catch (KeyStoreException | IOException | NoSuchAlgorithmException | CertificateException ex) {
            throw new RuntimeException(ex.getMessage());
        }
    }

    @Override
    public String getHashAlgorithm() {
        return DigestAlgorithms.getDigest(DigestAlgorithms.getAllowedDigest("SHA256"));
    }

    @Override
    public String getEncryptionAlgorithm() {
        try {
            PrivateKey pk = (PrivateKey)keyStore.getKey(alias, pin.toCharArray());
            return pk.getAlgorithm();
        } catch (KeyStoreException | NoSuchAlgorithmException | UnrecoverableKeyException ex) {
            throw new RuntimeException(ex.getMessage());
        }
    }

    public Certificate[] getChain() {
        try {
            return keyStore.getCertificateChain(alias);
        } catch (KeyStoreException ex) {
            throw new RuntimeException(ex.getMessage());
        }
    }

    @Override
    public byte[] sign(byte[] sh) throws GeneralSecurityException {
        PrivateKey pk = (PrivateKey)keyStore.getKey(alias, pin.toCharArray());
        String signMode = getHashAlgorithm();
        signMode += "with" + pk.getAlgorithm();
        Signature signature = Signature.getInstance(signMode, sunPKCS11);
        signature.initSign(pk);
        signature.update(sh);
        return signature.sign();
    }
}
